/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.ibm.j2ca.sample.twineball.inbound;

import java.rmi.RemoteException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.logging.Level;
import javax.resource.ResourceException;
import javax.resource.cci.Record;
import javax.resource.spi.CommException;
import javax.transaction.xa.Xid;
import com.ibm.j2ca.base.exceptions.BusinessObjectDefinitionNotFoundException;
import com.ibm.j2ca.extension.eventmanagement.Event;
import com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid;
import com.ibm.j2ca.extension.eventmanagement.XidImpl;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.ObjectNaming;
import com.ibm.j2ca.sample.twineball.TwineBallResourceAdapter;
import com.ibm.j2ca.sample.twineball.TwineBallStructuredRecord;
import com.ibm.j2ca.sample.twineball.outbound.commands.TwineBallCommandFactory;
import com.ibm.wbia.TwineBall.Server.TwineBallConnectionFactory;
import com.ibm.wbia.TwineBall.Server.TwineBallException;
import com.ibm.wbia.TwineBall.Server.TwineBallInterface;

/**
 * TwineBallEventStore This class implements polling for the TwineBallAdapter.
 */
public class TwineBallEventStoreWithXid implements EventStoreWithXid {

	private static final String EVENT_STATUS = "EVENT_STATUS="; //$NON-NLS-1$
	String eventTableName;
	TwineBallInterface connection;
	TwineBallCommandFactory commandFactory;
	TwineBallResourceAdapter resourceAdapter;
	LogUtils logger;
	ObjectNaming objectNaming;
	String namespace;
	TwineBallActivationSpecWithXid activationSpec;

	/**
	 * Create an eventStore instance.
	 * 
	 * @param activationSpec -
	 *            inbound properties
	 * @param resourceAdapter -
	 *            global properties, log/trace
	 * @throws ResourceException
	 */
	public TwineBallEventStoreWithXid(TwineBallActivationSpecWithXid activationSpec, TwineBallResourceAdapter resourceAdapter) throws ResourceException {
		this.activationSpec = activationSpec;
		// retrievalConnection = new ArrayList();
		eventTableName = activationSpec.getEventTableName();
		logger = resourceAdapter.getLogUtils();
		namespace = activationSpec.getBONamespace();
		logger.trace(Level.FINEST, TwineBallEventStoreWithXid.class.getName(), "Constructor", "Using event table: " + eventTableName);
		try {
			connection = new TwineBallConnectionFactory().getConnection(activationSpec.getUserName(), activationSpec.getPassword(), activationSpec.getSchema(), activationSpec.getUrl());
			connection.enableTracing(false);
		} catch (TwineBallException e) {
			logConnectFailed(activationSpec.url);
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logConnectFailed(activationSpec.url);
			logCommFailure("Constructor");
			throw new CommException(e);
		}
		objectNaming = new ObjectNaming(resourceAdapter);
		commandFactory = new TwineBallCommandFactory(objectNaming);
		this.resourceAdapter = resourceAdapter;
	}

	/**
	 * get either pending or in-progress events.
	 * 
	 * @param pollQuantity
	 *            -the number of events to poll
	 * @param status -
	 *            the status value we want
	 * @param eventTypeFilter -
	 *            event types.
	 * @return - an ArrayList of events
	 * @throws ResourceException
	 * @throws CommException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#getEvents(int, int, java.lang.String[])
	 */
	public ArrayList getEvents(int pollQuantity, int status, String[] eventTypeFilter) throws ResourceException, CommException {
		traceMethodEntrance("getEvents()");
		printEventStoreContents();
		ArrayList currentEvents = new ArrayList();
		Vector searchFields = new Vector();
		String searchString = EVENT_STATUS + status;
		searchFields.add(searchString);
		int currentFetchQuantity = 0;
		try {
			TwineBallInterface connectionForThisThread = new TwineBallConnectionFactory().getConnection(activationSpec.getUserName(), activationSpec.getPassword(), activationSpec.getSchema(), activationSpec.getUrl());
			Vector foundKeys = connectionForThisThread.findObjectKeysByContent(eventTableName, searchFields);
			logger.trace(Level.FINE, TwineBallEventStoreWithXid.class.getName(), "getEvents()", foundKeys.size() + " events found"); // CR 71244
			currentFetchQuantity = Math.min(pollQuantity, foundKeys.size());
			Iterator keyIterator = foundKeys.iterator();
			while (keyIterator.hasNext()) {
				String keyString = (String) keyIterator.next();
				Event currentEvent = getSpecificEvent(keyString);
				currentEvents.add(currentEvent);
			}
			connectionForThisThread.disconnect();
		} catch (TwineBallException e) {
			logGetEventsFailed(e);
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logGetEventsFailed(e);
			throw new CommException(e);
		}
		ArrayList sortedList = sortEvents(currentEvents);
		List subList = sortedList.subList(0, currentFetchQuantity);
		ArrayList returnableList = new ArrayList(currentFetchQuantity);
		returnableList.addAll(subList);
		traceFiner("getEvents()", subList.size() + " events will be processed"); // CR 71244
		traceMethodExit("getEvents()");
		return returnableList;
	}

	/**
	 * delete the event specified by evt.eventID
	 * 
	 * @param evt -
	 *            the event to be deleted
	 * @throws ResourceException
	 * @throws CommException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#deleteEvent(com.ibm.j2ca.extension.eventmanagement.Event)
	 */
	public void deleteEvent(Event evt) throws ResourceException, CommException {
		traceMethodEntrance("deleteEvent()");
		try {
			traceFiner("deleteEvent()", "deleting event with ID" + evt.getEventId());
			connection.deleteObject("WBIA_EVENTS", "" + evt.getEventId());
		} catch (TwineBallException e) {
			logDeleteFailed(evt.getEventId());
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logDeleteFailed(evt.getEventId());
			logCommFailure("deleteEvent()");
			throw new CommException(e);
		}
		printEventStoreContents();
		traceMethodExit("deleteEvent()");
	}

	/**
	 * update the status of the event.
	 * 
	 * @param evt -
	 *            the event to update
	 * @throws ResourceException
	 * @throws CommException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#updateEventStatus(com.ibm.j2ca.extension.eventmanagement.Event, int)
	 */
	public void updateEventStatus(Event event, int newstatus) throws ResourceException, CommException {
		traceMethodEntrance("updateEventStatus()");
		traceFiner("updateEventStatus()", "updating event:" + event.getEventId() + " to status:" + newstatus);
		Vector updateValues = new Vector();
		String statusPair = EVENT_STATUS + newstatus;
		updateValues.addElement(statusPair);
		try {
			connection.updateObject(eventTableName, "" + event.getEventId(), updateValues);
		} catch (TwineBallException e) {
			logUpdateFailed(event.getEventId());
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logUpdateFailed(event.getEventId());
			logCommFailure("updateEventStatus()");
			throw new CommException(e);
		}
		printEventStoreContents();
		traceMethodExit("updateEventStatus()");
	}

	/**
	 * can we call commit?
	 * 
	 * @return
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#isTransactional()
	 */
	public boolean isTransactional() {
		return true;
	}

	/**
	 * Roll back the transaction
	 * 
	 * @throws ResourceException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#rollbackWork()
	 */
	public void rollbackWork() throws ResourceException {
		try {
			connection.rollback();
		} catch (TwineBallException e) {
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logCommFailure("rollbackWork()");
			throw new CommException(e);
		}
	}

	/**
	 * commit the transaction
	 * 
	 * @throws ResourceException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#commitWork()
	 */
	public void commitWork() throws ResourceException {
		try {
			connection.commit();
		} catch (TwineBallException e) {
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logCommFailure("commitWork()");
			throw new CommException(e);
		}
	}

	/**
	 * Get the value in a particular column.
	 * 
	 * @param eventRecord
	 * @param columnName
	 * @return
	 */
	private String getColumnData(Vector eventRecord, String columnName) {
		int vectorPosition = getElementNumber(eventRecord, columnName);
		String currentElement = (String) eventRecord.elementAt(vectorPosition);
		String columnData = (currentElement.substring(currentElement.indexOf('=') + 1));
		return columnData;
	}

	private int getElementNumber(Vector eventRecord, String columnName) {
		for (int i = 0; i < eventRecord.size(); i++) {
			String currentElement = (String) eventRecord.elementAt(i);
			String currentColumnName = currentElement.substring(0, currentElement.indexOf('='));
			if (columnName.equalsIgnoreCase(currentColumnName)) { // found it
				return i;
			}
		}
		return 0;
	}

	/**
	 * get the event specified by eventID.
	 * 
	 * @param eventId
	 * @return the event that we're interested in.
	 * @throws ResourceException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#getSpecificEvent(java.lang.String)
	 */
	public Event getSpecificEvent(String eventId) throws ResourceException {
		traceMethodEntrance("getSpecificEvent()");
		traceFiner("getSpecificEvent()", "getting event with ID: " + eventId);
		TwineBallEvent currentEvent;
		try {
			Vector currentObject = connection.getObject(eventTableName, eventId);
			if (currentObject == null || currentObject.isEmpty()) {
				throw new TwineBallException("Unable to retrieve event from Event table");
			}
			String currentObjectName = getColumnData(currentObject, "Object_Name");
			currentEvent = new TwineBallEvent(getColumnData(currentObject, "primaryKey"), currentObjectName, null, Event.NEWEVENT);
			currentEvent.setEventKeys(getColumnData(currentObject, "Object_Key"));
			currentEvent.setVerb(getColumnData(currentObject, "Object_Verb"));
			String timestampData = getColumnData(currentObject, "Event_Time");
			Timestamp t = Timestamp.valueOf(timestampData);
			currentEvent.setTimeStamp(t);
		} catch (TwineBallException e) {
			logGetSpecificEventFailed(eventId);
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logGetSpecificEventFailed(eventId);
			logCommFailure("getSpecificEvent()");
			throw new CommException(e);
		}
		traceMethodExit("getSpecificEvent()");
		return currentEvent;
	}

	/**
	 * For this event, get the complete TwineBall Object that it represents.
	 * 
	 * @param event
	 * @return
	 * @throws ResourceException
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStore#getObjectForEvent(com.ibm.j2ca.extension.eventmanagement.Event)
	 */
	// public synchronized Record getRecordForEvent(Event event) throws ResourceException {
	// TwineBallEvent tbEvent = (TwineBallEvent) event;
	// traceMethodEntrance("getObjectForEvent()");
	// traceFiner("getObjectForEvent()", "attempting to retrieve full object for event:" + event);
	// String type = event.getEventType();
	// traceFiner("getObjectForEvent()", "object type:" + type);
	// DataObject wrapperObject;
	// String targetnamespace = namespace + "/"+ type.toLowerCase() + "bg";
	// type = type + "BG";
	// try {
	// Type type = TypeFactory(targetnamespace, type);
	// wrapperObject = AdapterBOUtil.createDataObject(targetnamespace, type);
	// } catch (BusinessObjectDefinitionNotFoundException e) {
	// logBONotFound(e);
	// throw e;
	// }
	// //DataObject dataObject = wrapperObject.createDataObject(AdapterBOUtil.getRootBusinessObjectProperty(wrapperObject.getType()));
	// objectNaming.setKey(dataObject, event.getEventKeys());
	// Command retrieveCommand = commandFactory.createCommand(WBIInteractionSpec.RETRIEVE_OP, dataObject);
	// retrieveCommand.setConnection(connection);
	// retrieveCommand.setLogUtils(logger);
	// DataObject returnedObject = retrieveCommand.execute(dataObject);
	// WBIRecord outputRecord = new WBIRecord();
	// wrapperObject.setDataObject(AdapterBOUtil.getRootBusinessObjectProperty(wrapperObject.getType()), returnedObject);
	// traceFiner("getObjectForEvent()", "setting verb:" + tbEvent.getVerb());
	// wrapperObject.set("verb", tbEvent.getVerb());
	// outputRecord.setDataObject(wrapperObject);
	// traceMethodEntrance("getObjectForEvent()");
	// return outputRecord;
	// return null;
	// }
	public synchronized Record getRecordForEvent(Event event) throws ResourceException {
		TwineBallStructuredRecord outputRecord = null;
		try {
			TwineBallEvent tbEvent = (TwineBallEvent) event;
			traceMethodEntrance("getObjectForEvent()");
			traceFiner("getObjectForEvent()", "attempting to retrieve full object for event:" + event);
			String typename = event.getEventType();
			String key = event.getEventKeys();
			Vector contents = connection.getObject(typename, key);
			outputRecord = new TwineBallStructuredRecord();
			outputRecord.setEISRepresentation(contents);
			outputRecord.setRecordName(typename);
			outputRecord.setTwineBallConnection(connection);
			outputRecord.setOperationName(tbEvent.getVerb());
			if (tbEvent.getVerb().equalsIgnoreCase("Delete")) {
				outputRecord.setPrimaryKey(tbEvent.getEventKeys());
			}
		} catch (Exception e) {
			throw new ResourceException(e);
		}
		return outputRecord;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#getEventForXid(com.ibm.j2ca.extension.eventmanagement.XidImpl)
	 */
	public Event getEventForXid(XidImpl xid) throws ResourceException, CommException {
		traceMethodEntrance("getEventForXid");
		printEventStoreContents();
		Vector searchPairs = new Vector();
		String xidPair = "XID=" + xid;
		searchPairs.addElement(xidPair);
		Vector keys = null;
		try {
			keys = connection.findObjectKeysByContent(eventTableName, searchPairs);
			if (keys.size() > 0) {
				traceFine("getEventForXid()", "Keys found: " + keys.size());
				logGetEventForXidFailed(xid.toString());
				throw new ResourceException("Too many Xids matched");
			}
		} catch (TwineBallException e) {
			logGetEventForXidFailed(xid.toString());
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logGetEventForXidFailed(xid.toString());
			logCommFailure("getEventForXid");
			throw new CommException(e);
		}
		String eventID = (String) keys.get(0);
		Event event = getSpecificEvent(eventID);
		traceMethodExit("getEventForXid");
		return event;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#getPendingTransactions()
	 */
	public Xid[] getPendingTransactions() throws ResourceException, CommException {
		traceMethodEntrance("getPendingTransactions");
		printEventStoreContents();
		Vector searchFields = new Vector();
		ArrayList xids = new ArrayList();
		String searchString = EVENT_STATUS + Event.NEWEVENT;
		searchFields.add(searchString);
		try {
			Vector foundKeys = connection.findObjectKeysByContent(eventTableName, searchFields);
			Iterator keyIterator = foundKeys.iterator();
			while (keyIterator.hasNext()) {
				String keyString = (String) keyIterator.next();
				Vector currentObject = connection.getObject(eventTableName, keyString);
				if (currentObject == null || currentObject.isEmpty()) {
					throw new ResourceException("Unable to retrieve event from Event table");
				}
				String xidString = getColumnData(currentObject, "Xid");
				// CR85505 start
				if (xidString != null && xidString != "" && !xidString.equals("null") && !xidString.equals(null)) {
					XidImpl xid = new XidImpl(xidString);
					xids.add(xid);
				}
				// CR85505 end
			}
		} catch (TwineBallException e) {
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logCommFailure("getEventForXid");
			throw new CommException(e);
		}
		Xid[] xidArray = new Xid[xids.size()];
		for (int i = 0; i < xidArray.length; i++) {
			xidArray[i] = (XidImpl) xids.get(i);
		}
		traceFine("getPendingTransactions", "found " + xidArray.length + " pending transactions");
		traceMethodExit("getPendingTransactions");
		return xidArray;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#setEventTransactionID(com.ibm.j2ca.extension.eventmanagement.Event, com.ibm.j2ca.extension.eventmanagement.XidImpl)
	 */
	public void setEventTransactionID(Event event, XidImpl xid) throws ResourceException, CommException {
		traceMethodEntrance("setEventTransactionID");
		traceFiner("updateEventStatus()", "assoiating event:" + event.getEventId() + " to xid:" + xid);
		Vector updateValues = new Vector();
		String xidPair = "XID=" + xid;
		updateValues.addElement(xidPair);
		try {
			connection.updateObject(eventTableName, "" + event.getEventId(), updateValues);
		} catch (TwineBallException e) {
			logUpdateFailed(event.getEventId());
			throw new ResourceException(e);
		} catch (RemoteException e) {
			logUpdateFailed(event.getEventId());
			logCommFailure("setEventTransactionID");
			throw new CommException(e);
		}
		printEventStoreContents();
		traceMethodExit("setEventTransactionID");
	}

	/**
	 * Given a list of events, sort them by date.
	 * 
	 * @param currentEvents
	 * @return
	 */
	private ArrayList sortEvents(ArrayList currentEvents) {
		ArrayList sortedEvents = (ArrayList) currentEvents.clone();
		Collections.sort(sortedEvents, new Comparator() {

			public int compare(Object event1, Object event2) {
				return isAfter((Event) event1, (Event) event2);
			}
		});
		return sortedEvents;
	}

	private int isAfter(Event event1, Event event2) {
		if (event1.getTimeStamp().after(event2.getTimeStamp()))
			return 1;
		return 0;
	}

	public boolean implementsFiltering() {
		return false;
	}

	private void traceMethodEntrance(String methodName) {
		logger.traceMethodEntrance(TwineBallEventStoreWithXid.class.getName(), methodName);
	}

	private void traceMethodExit(String methodName) {
		logger.traceMethodExit(TwineBallEventStoreWithXid.class.getName(), methodName);
	}

	private void traceFine(String methodName, String message) {
		logger.trace(Level.FINE, TwineBallEventStoreWithXid.class.getName(), methodName, message);
	}

	private void traceFiner(String methodName, String message) {
		logger.trace(Level.FINER, TwineBallEventStoreWithXid.class.getName(), methodName, message);
	}

	private void traceFinest(String methodName, String message) {
		logger.trace(Level.FINEST, TwineBallEventStoreWithXid.class.getName(), methodName, message);
	}

	private void logGetEventsFailed(Exception e) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "getEvents()", "getEvents() failed.  Exception:" + e.getClass().getName() + ":" + e.getMessage());
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "getEvents()", "1002", new Object[] {});
	}

	private void logDeleteFailed(String eventID) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "deleteEvent()", "delete failed for event id:" + eventID);
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "deleteEvent()", "1003", new Object[] { eventID });
	}

	private void logUpdateFailed(String eventID) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "updateEvent()", "update failed for event id:" + eventID);
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "updateEvent()", "1006", new Object[] { eventID });
	}

	private void logGetSpecificEventFailed(String eventID) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "getSpecificEvent()", "getSpecificEvent failed for event id:" + eventID);
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "getSpecificEvent()", "1006", new Object[] { eventID });
	}

	private void logGetEventForXidFailed(String xid) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "getEventForXid()", "getEventForXid() failed for xid:" + xid);
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "getEventForXid()", "1006", new Object[] { xid });
	}

	private void logCommFailure(String methodName) {
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), methodName, "1004", new Object[] {});
	}

	private void logConnectFailed(String url) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "Constructor", "unable to connect to TwineBall at URL:" + url);
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "getEvents()", "1002", new Object[] {});
	}

	private void logBONotFound(BusinessObjectDefinitionNotFoundException e) {
		logger.trace(Level.SEVERE, TwineBallEventStoreWithXid.class.getName(), "getObjectForEvent()", "unable to create business object for type" + e.getNamespace() + "/" + e.getType());
		logger.log(Level.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, TwineBallEventStoreWithXid.class.getName(), "getObjectForEvent()", "1009", new Object[] { e.getNamespace() + "/" + e.getType() });
	}

	private void printEventStoreContents() {
		if (logger.isTraceEnabled(Level.FINEST)) {
			Vector searchPairs = new Vector();
			searchPairs.add("Object_Name=Customer");
			try {
				Vector keys = connection.findObjectKeysByContent("WBIA_EVENTS", searchPairs);
				traceFinest("printEventStore", "Printing eventStore contents.  EventStore contains: " + keys.size() + " events.");
				Iterator keyIterator = keys.iterator();
				while (keyIterator.hasNext()) {
					String key = (String) keyIterator.next();
					Vector record = connection.getObject("WBIA_EVENTS", key);
					traceFinest("printEventStore", "WBIA_EVENTS record" + record);
				}
				connection.commit();
			} catch (TwineBallException e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#close()
	 */
	public void close() {
		// TODO Auto-generated method stub
	}
}
