/*
 * Licensed Materials - Property of IBM
 * 5724-G93
 * (C) Copyright IBM Corporation 2003. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* DISCLAIMER:
 * The following [enclosed] code is sample code created by IBM
 * Corporation. This sample code is not part of any standard IBM product
 * and is provided to you solely for the purpose of assisting you in the
 * development of your applications. The code is provided 'AS IS',
 * without warranty of any kind IBM shall not be liable for any damages
 * arising out of your use of sample code, even if they have been
 * advised of the possibility of such damages.
 */

package com.ibm.wbia.TwineBall.Server;

/**
 * @author Jim Haungs, Travis Nelson  
 */

import java.util.*;
import java.rmi.*;


public class TwineBallConnectionPool {

	private TwineBallConnectionFactory server;
	private Hashtable connections;
	private ArrayList freeList;
	private String login;
	private String password;
	private String schemaName;
	private String URL;
	private boolean tracing;
	

	public TwineBallConnectionPool(String login, String password, String schemaName, String URL) throws TwineBallException {
		this.login = login;
		this.password = password;
		this.tracing = true;
		this.schemaName = schemaName;
		this.URL = URL;
		server = new TwineBallConnectionFactory();
		initialize();		
	}

	private void initialize() {
		connections = new Hashtable();
		freeList = new ArrayList();
	}

	public synchronized TwineBallInterface getConnection() throws RemoteException, TwineBallException {
		TwineBallInterface connection;

		Thread thread = Thread.currentThread();
		connection = connectionFor(thread);

		if (connection != null)
			return connection;

		if (freeList.isEmpty())
			connection = newConnection();
		else {
			connection = (TwineBallInterface) freeList.remove(0);
			trace("Server #" + connection.getClientID() + " used from free list [" + freeList.size() + "]");
		}

		connections.put(thread, connection);
		trace("Server #" + connection.getClientID() + " added to connection pool [" + connections.size() + "]");

		return connection;
	}

	public synchronized void release(TwineBallInterface connection) throws RemoteException, TwineBallException {
		Thread thread = Thread.currentThread();
		TwineBallInterface myConnection = connectionFor(thread);

		if (connection != myConnection)
			throw new TwineBallException("Attempt to release a connection owned by a different thread.");

		connections.remove(thread);
		freeList.add(myConnection);

		trace("Server #" + myConnection.getClientID() + " moved to free list [" + freeList.size() + "]");
	}

	public synchronized int closeAllConnections() throws RemoteException, TwineBallException {
		int closed;

		closed = closeReleasedConnections();
		closed += closeConnections(new ArrayList(connections.keySet()));

		trace(closed + " connection" + (closed == 1 ? "" : "s") + " closed.");

		initialize();

		return closed;
	}

	private int closeReleasedConnections() throws RemoteException, TwineBallException {
		int count = 0;

		for (Iterator iter = freeList.iterator(); iter.hasNext();) {
			TwineBallInterface connection = (TwineBallInterface) iter.next();

			if (connection.connected()) {
				connection.disconnect();
				count++;
			}
		}

		return count;
	}

	private int closeConnections(ArrayList threads) throws RemoteException, TwineBallException {
		int closed = 0;

		for (Iterator iter = threads.iterator(); iter.hasNext();) {
			Thread thread = (Thread) iter.next();
			TwineBallInterface connection = connectionFor(thread);

			if (connection.connected()) {
				connection.disconnect();
				closed++;
			}

		}

		return closed;
	}

	public void enableTracing(boolean flag) {
		tracing = flag;
	}

	public synchronized int availableConnections() {
		return freeList.size();
	}

	public synchronized int size() {
		return connections.size();
	}

	private TwineBallInterface connectionFor(Thread thread) {
		return (TwineBallInterface) connections.get(thread);
	}

	private TwineBallInterface newConnection() throws TwineBallException {
		TwineBallInterface connection = null;

		try {
			connection = server.getConnection(login, password, schemaName, URL);
			connection.enableTracing(tracing);
			trace("Connected to server #" + connection.getClientID());
		} catch (Exception ex) {
			ex.printStackTrace();
			throw new TwineBallException("Unable to connect to server: " + ex.getMessage());
		}

		return connection;
	}

	private void trace(String message) {
		if (tracing)
			System.out.println(message);
	}
}
