package com.ibm.wbia.TwineBall.Server;

import com.ibm.wbia.TwineBall.Server.*;
import junit.framework.*;
import java.rmi.*;
import java.util.*;
import java.sql.*;

/**
 * @author jhaungs
 */
public class TwineBallServerTest
extends TestCase
{
	private TwineBallInterface connection;
	private String singleKey;

	public TwineBallServerTest (String name)
	{
		super (name);
		}

	protected void setUp()
	throws Exception
	{
		Properties properties = new TwineBallProperties().getProperties();  
		TwineBallConnectionFactory factory = new TwineBallConnectionFactory();
		String url = properties.getProperty("server.name");
		if (url == null) {
			url = "";
		}
		connection = factory.getConnection("twineball","twineball","twineball",url);
		connection.enableTracing (true);
		System.out.println ("\nTest " + getName() + " connected to server #" + connection.getClientID());
		singleKey = insertTestingObject ("Customer", customer());
		}

	protected void tearDown() throws Exception
	{
		if (singleKey != null)
			connection.deleteObject("Customer", singleKey);

		if (connection != null) {
			connection.commit();
			connection.disconnect();
			}

		System.out.println ("Test " + getName() + " finished.\n");
		}

	public String insertTestingObject(String entityName, Vector entity)
	throws TwineBallException, RemoteException
	{
		return connection.insertObject(entityName, entity);
		}

	public Vector customer()
	{
	Vector customer = new Vector (9);

		customer.addElement ("CUSTOMERTYPE=Customer");
		customer.addElement ("FIRSTNAME=Bart");
		customer.addElement ("LASTNAME=Simpson");
		customer.addElement ("MIDDLEINITIAL=Q");
		customer.addElement ("TITLE=Sir");
		customer.addElement ("SEX=M");
		customer.addElement ("PRIMARYPHONE=555-5555");
		customer.addElement ("SECONDARYPHONE=123-4567");

		return customer;
		}

	public void testInsertObject()
	throws TwineBallException, RemoteException
	{
		Vector customer2 = connection.getObject("Customer", singleKey);
		assertTrue (customer2.contains("PRIMARYKEY=" + singleKey));
		}

	public void testDeleteObject()
	throws TwineBallException, RemoteException
	{
		Vector customer = connection.getObject("Customer", singleKey);
		assertFalse (customer.isEmpty());

		connection.deleteObject ("Customer", singleKey);
		customer = connection.getObject("Customer", singleKey);
		assertTrue (customer.isEmpty());
	}

	public void testNestedObject()
	throws TwineBallException, RemoteException
	{
		Vector customer2 = connection.getObject("Customer", singleKey);
		assertFalse (customer2.isEmpty());

		String addressKey1 = insertAddress (singleKey, newAddress (101, "Mailing"));
		String addressKey2 = insertAddress (singleKey, newAddress (102, "Billing"));

		Vector keys = connection.findChildKeys("Customer", singleKey, "Address");
		printVector ("Key", keys);
		assertTrue(keys.size() == 2);
		assertTrue(keys.contains(addressKey1));
		assertTrue(keys.contains(addressKey2));

		connection.deleteObject ("Customer", singleKey);
		customer2 = connection.getObject("Customer", singleKey);
		assertTrue (customer2.isEmpty());

		Vector address = connection.getObject("Address", addressKey1);
		assertTrue (address.isEmpty());
		address = connection.getObject("Address", addressKey2);
		assertTrue (address.isEmpty());

	}

	private String insertAddress(String parentKey, Vector address)
	throws RemoteException, TwineBallException
	{
		address.addElement("PARENTKEY=" + parentKey);
		address.addElement("PARENTTABLE=CUSTOMER");

		return connection.insertObject("Address", address);
		}

	private Vector newAddress(int index, String addressType)
	throws RemoteException, TwineBallException
	{
	Vector address = new Vector();

		address.addElement("ADDRESSTYPE=" + addressType);
		address.addElement("ADDRESSLINE1=" + index + " Main Street");
		address.addElement("CITY=San Francisco");
		address.addElement("STATE=CA");
		address.addElement("COUNTRY=USA");
		address.addElement("ZIPCODE=94131");

		return address;
		}

	public void testGetObject()
	throws Exception, SQLException
	{
	Vector customer = connection.getObject ("Customer", singleKey);

		assertTrue (customer.size() >= 8);
		assertTrue (customer.contains("PRIMARYKEY=" + singleKey));
		assertTrue (customer.contains("CUSTOMERTYPE=Customer"));
		assertTrue (customer.contains("FIRSTNAME=Bart"));
		assertTrue (customer.contains("LASTNAME=Simpson"));
		assertTrue (customer.contains("MIDDLEINITIAL=Q"));
		assertTrue (customer.contains("TITLE=Sir"));
		assertTrue (customer.contains("SEX=M"));
		assertTrue (customer.contains("PRIMARYPHONE=555-5555"));
		assertTrue (customer.contains("SECONDARYPHONE=123-4567"));

		printVector ("Customer", customer);
		}

	public void testFindAllObjectKeys()
	throws TwineBallException, RemoteException
	{
		Vector searchPairs = new Vector();
		Vector keys = connection.findObjectKeysByContent("Customer", searchPairs);
		assertTrue(keys.contains(singleKey));
		}


	public void testFindObjectsByKeyContent()
	throws TwineBallException, RemoteException
	{
		Vector searchPairs = new Vector();
		searchPairs.add ("primarykey="+singleKey);

		Vector keys = connection.findObjectKeysByContent("Customer", searchPairs);
		assertTrue(keys.size() == 1);
		assertTrue(keys.contains(singleKey));
		}

	public void testFindObjectKeysByValue()
		throws TwineBallException, RemoteException
		{
			Vector searchPairs = new Vector();
			searchPairs.add ("firstname=Bart");
			searchPairs.add ("lastname=Simpson");

			Vector keys = connection.findObjectKeysByContent("Customer", searchPairs);
			assertTrue(keys.contains(singleKey));
			}

	public void testGetEntityNames ()
	throws Exception, SQLException
	{
	Vector names = connection.getEntityNames();

		assertTrue (names.contains("ADDRESS"));
		assertTrue (names.contains("CUSTOMER"));
		assertTrue (names.contains("PARENTCHILD"));

		printVector ("Table", names);
		}

	public void testGetRootNames ()
	throws Exception, SQLException
	{
	Vector names = connection.getRootObjects();

		printVector ("Table", names);
		assertTrue (names.contains("CUSTOMER"));
		}

	public void testGetHierarchy ()
	throws Exception, SQLException
	{
	Vector parents = connection.getRootObjects();

		for (Enumeration p = parents.elements(); p.hasMoreElements();) {
			String parent = p.nextElement().toString();
			Vector children = connection.getChildNames (parent);

			for (Enumeration c = children.elements(); c.hasMoreElements();) {
				String child = c.nextElement().toString();
				System.out.println (parent + " -> " + child);
				}
			}
		}

	public void testNameConversions()
	{
		assertTrue ("OrderItem".equals (CaseChange.toMixedCase("ORDER_ITEM")));
		assertTrue ("OI"       .equals (CaseChange.toMixedCase("O_I")));
		assertTrue ("O"        .equals (CaseChange.toMixedCase("O")));
		assertTrue ("Orderitem".equals (CaseChange.toMixedCase("ORDERITEM")));

		assertTrue ("ORDER_ITEM".equals (CaseChange.toUpperCase("OrderItem")));
		assertTrue ("OR_I"      .equals (CaseChange.toUpperCase("OrI")));
		assertTrue ("O"         .equals (CaseChange.toUpperCase("O")));
		assertTrue ("O"         .equals (CaseChange.toUpperCase("O")));
		assertTrue ("ADDRESS"   .equals (CaseChange.toUpperCase("Address")));
		}

	public void testGetColumnNames ()
	throws Exception, SQLException
	{
	Vector names = connection.getColumnNames("Customer");

		assertTrue (names.size() >= 8);

		assertTrue (names.contains("PRIMARYKEY"));
		assertTrue (names.contains("CUSTOMERTYPE"));
		assertTrue (names.contains("FIRSTNAME"));
		assertTrue (names.contains("LASTNAME"));
		assertTrue (names.contains("MIDDLEINITIAL"));
		assertTrue (names.contains("TITLE"));
		assertTrue (names.contains("SEX"));
		assertTrue (names.contains("PRIMARYPHONE"));
		assertTrue (names.contains("SECONDARYPHONE"));

		printVector ("Name", names);
		}

	public void testGetColumnWidths ()
	throws Exception, SQLException
	{
		assertTrue (connection.getColumnWidth ("Customer", "PRIMARYKEY") == 10);
		assertTrue (connection.getColumnWidth ("Customer", "FIRSTNAME") == 20);
		assertTrue (connection.getColumnWidth ("Customer", "LASTNAME") == 20);
		assertTrue (connection.getColumnWidth ("Customer", "MIDDLEINITIAL") == 1);
		assertTrue (connection.getColumnWidth ("Customer", "TITLE") == 10);
		assertTrue (connection.getColumnWidth ("Customer", "SEX") == 1);
		assertTrue (connection.getColumnWidth ("Customer", "PRIMARYPHONE") == 20);
		assertTrue (connection.getColumnWidth ("Customer", "SECONDARYPHONE") == 20);
		}

	public void testGetColumnTypes ()
	throws Exception, SQLException
	{
	Vector columns = connection.getColumnNames("Customer");

		for (Enumeration e = columns.elements(); e.hasMoreElements();) {
			String name = (String) e.nextElement();
			String type = connection.getColumnType("Customer", name);
			System.out.println ("Column " + name + " type = " + type);
			//assertTrue (type.startsWith ("VARCHAR"));
			}
		}

	public void testGetParentNames()
		throws Exception, SQLException
	{
	Vector parents;

		parents = connection.getParentNames("Address");
		printVector ("Parent", parents);
		assertTrue (parents.contains("CUSTOMER"));
		}

	public void testGetChildNames()
	throws Exception, SQLException
	{
	Vector children;

		children = connection.getChildNames("Customer");
		assertTrue (children.contains("ADDRESS"));

		printVector ("Child", children);
		}

	private void printVector (String name, Vector vector)
	{
		System.out.println (name + " vector [" + vector.size() + "]");

		for (Enumeration e = vector.elements(); e.hasMoreElements();) {
			System.out.println (name + ": " + e.nextElement().toString());
			}
		}
	}
